/**
********************************************************************************
Copyright 2012 Ortus Solutions, Corp
www.ortussolutions.com
********************************************************************************
Main Databoss Handler
*/
component{

	// Dependencies
	property name="entityService"    inject="model:entityService@databoss";
	property name="metadataService"  inject="model:metadataService@databoss";
	property name="entityDisplayHelper"  inject="model:entityDisplayHelper@databoss";
	property name="i18n"			 inject="coldbox:plugin:i18n";
	property name="logger"			 inject="logbox:logger:databoss";

	/**
	* preHandler
	*/
	function preHandler( event, rc, prc, action ){
		// Get Module Settings
		var moduleSettings = getModuleSettings( 'Databoss' ).settings;
		// Param format values
		event.paramValue( "format", "html" );
		event.paramValue( "rURL", "" );
		rc.entityDisplayHelper = entityDisplayHelper;
		
		// Global Exit Handlers
		prc.xehEntityList 		= "databoss/entity";
		prc.xehDataboss			= "databoss";
		prc.xehDictionaryClean  = "databoss/cleanDictionary";
		prc.xehReloadORM		= "databoss/reloadORM";
		prc.xehReloadApp		= "databoss/reloadApp";

		// Get correct paging plugin if in module mode
		prc.oPaging = getModel( "paging@databoss" );
		// Determine correct module root of operation
		prc.modRoot = event.getModuleRoot( "databoss" ) & "/";
		
		// check for locale changes
		if( event.valueExists( 'locale' ) ){
			// validate locale
			if( arrayFindNoCase( moduleSettings.supportedLanguages, rc.locale ) gt 0 ){
				i18n.setFwLocale( rc.locale );
			}
		}
		prc.userLocale = i18n.getFwLocale();
		
		// Load System Entities
		prc.systemEntities 		= metadataService.getPersistedEntities();
		prc.systemDictionary 	= metadataService.getDictionary();

		// Param Incoming Entity Values
		if( arrayLen( prc.systemEntities ) ){
			event.paramValue( "entity", prc.systemEntities[ 1 ] );
			rc.entity 		= URLDecode( rc.entity );
			rc.safeEntity 	= urlEncodedFormat( rc.entity );
		}
		
		// Only load assets for HTML
		if( rc.format eq "html" ){
			// Load CSS
			addAsset( "#prc.modRoot#includes/styles/bootstrap.min.css,
					   #prc.modRoot#includes/styles/bootstrap-theme.min.css,
					   #prc.modRoot#includes/styles/databoss-datepicker.css,
					   #prc.modRoot#includes/styles/databoss-timepicker.css,
					   #prc.modRoot#includes/styles/databoss-main.css,
					   #prc.modRoot#includes/styles/databoss-wysihtml5-0.0.2.css" );
			// Load JS
			if( moduleSettings.loadJquery ){
				addAsset( "#prc.modRoot#includes/js/jquery-1.10.2.min.js" );
			}
			addAsset( "#prc.modRoot#includes/js/bootstrap.min.js,
					   #prc.modRoot#includes/js/databoss.modernizr.js,
					   #prc.modRoot#includes/js/databoss.datepicker.js,
					   #prc.modRoot#includes/js/databoss.timepicker.js,
					   #prc.modRoot#includes/js/databoss.metadata.pack.js,
					   #prc.modRoot#includes/js/databoss.tablesorter.min.js,
					   #prc.modRoot#includes/js/databoss.uitablefilter.js,
					   #prc.modRoot#includes/js/wysihtml5-0.3.0_rc2.min.js,
					   #prc.modRoot#includes/js/databoss.wysihtml5-0.0.2.min.js" );
		}
	}

	/**
	* display listings for entities
	*/
	function index(event, rc, prc){
		// Validate we have entities to show
		if( ArrayLen( prc.systemEntities ) ){
			// params
			event.paramValue( "showAll", false );
			
			// exit handlers
			prc.xehEntityDelete = "databoss/doDelete";

			//Get Entity's Meta Data Dictionary
			prc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );

			// Calculate the start row
			prc.pagingBoundaries = prc.oPaging.getBoundaries();
			prc.pagingLink = event.buildLink( prc.xehEntityList ) & "/#rc.safeEntity#/page/@page@";
			
			//Get entity listing
			prc.listingCount = entityService.count(entityName=rc.entity);
			prc.aListing = entityService.list(entityName=rc.entity,
											  sortOrder=prc.mdDictionary.sortBy,
											  offset=( rc.showAll ? 0 : prc.pagingBoundaries.startRow ),
											  max=( rc.showAll ? 0 : prc.oPaging.getPagingMaxRows() ),
											  asQuery=false);
		}
		else{
			rc.entity = rc.safeEntity = "";
		}

		// Rendering Formats
		prc.export = false;
		switch( rc.format ){
			case "json" : case "xml" : case "wddx" : case "jsonp" : {
				event.renderData(type=rc.format, 
								 data=entityToQuery( prc.aListing ), 
								 xmlRootName="#rc.entity#", 
								 jsonCallback=getModuleSettings("databoss").settings.jsonpcallback);
				
				break;
			}
			case "pdf" : {
				prc.export = true;
				event.renderData( type="pdf", data=renderView(view="databoss/index", module="databoss") );
				break;
			}
			default: {
				//Set view to render
				event.setView("databoss/index");	
			}
		}
		
	}

	/**
    * Cleans the entire metadata dictionary
	* @working true
	* @finalized false
    */
    function cleanDictionary(event, rc, prc){
		var nextEvent =	event.getValue('returnURL', 'databoss');

    	metadataService.cleanDictionary();
		
		getPlugin("MessageBox").setMessage("info", getResource('db_dictionary_cleaned'));
		setNextEvent(nextEvent);
    }

    /**
    * reload the ORM
    */
    function reloadORM(event, rc, prc){
		var nextEvent =	event.getValue('returnURL', 'databoss');
	
    	ORMREload();
    	metadataService.cleanDictionary();
    	
    	getPlugin("MessageBox").setMessage("info", getResource('db_orm_reloaded'));
		setNextEvent(nextEvent);
    }
    
    /**
    * reload the Application
    */
    function reloadAPP(event, rc, prc){
		var nextEvent =	event.getValue('returnURL', 'databoss');
		
		// Ran through post process
		getPlugin("MessageBox").setMessage("info", getResource('db_app_reloaded'));
		setNextEvent(nextEvent);
    }

	/**
    * Delete an entity from the system
    */
    function doDelete(event, rc, prc){
    	
		//Check that listing sent in
		if ( len( event.getTrimValue("entityID","") ) ){
			var numEntities = listLen( rc.entityID );
			var errors 		= [];
			//Loop through listing and delete objects
			for(var i=1; i lte numEntities; i++){
				try{
					entityService.deleteByID(entityName=rc.entity, id=listGetAt( rc.entityId, i) );
				}
				catch(any e){
					arrayAppend(errors, e.message & e.detail);
					log.error("Error deleting entity: #rc.safeEntity#(#listGetAt( rc.entityId, i)#) : #e.message# #e.detail#", e );
				}
			}
			
			// MessageBox
			if( arrayLen( errors ) eq 0 ){
				getPlugin("MessageBox").info( getResource('db_record_deletion_complete') );
			}
			else{
				var errorMsg = 	"";
				// create customized error message
				if(numEntities > 1){
					errorMsg =	getResource('db_record_deletion_multi_failed');
					errorMsg =	replace( replace(errorMsg, '{1}', arrayLen( errors ) ), '{2}', numEntities );
				}
				else{
					errorMsg =	getResource('db_record_deletion_failed');
				}
				errorMsg = errorMsg & getPlugin('HTMLHelper').ul( errors );
				getPlugin("MessageBox").error( errorMsg );
			}
		}
		else{
			// MessageBox
			getPlugin("MessageBox").warn( getResource('db_record_deletion_no_records') );
		}

		// If the user came from somewhere specific...
		if( len( rc.rURL ) ) {
			// ...go back there.
			setNextEvent( event=rc.rURL );
		} else {
			// Otherwise, relocate back to listing
			setNextEvent(event="databoss.entity", queryString=rc.safeEntity);
		}
    }

	/**
    * Create the creation form
    */
    function create(event, rc, prc){
		// Get Entity's md Dictionary
		prc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );
		// Get Default Entity
		prc.oEntity = entityService.new( prc.mdDictionary.name );
		//Check Singular Relations
		if ( prc.mdDictionary.hasSingularRelations ){
			//Get Entity Listings
			for (var i=1; i lte arrayLen( prc.mdDictionary.singularRelations ); i++ ){
				prc[ "q#prc.mdDictionary.singularRelations[i].alias#" ] = entityService.list( entityName=prc.mdDictionary.singularRelations[i].sourceEntity );
			}
		}

		//Set view.
		event.setView("databoss/create");
    }

	/**
    * doCreate
    */
    function doCreate(event, rc, prc){
		// Get Entity's md Dictionary
		prc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );
		// Get a new entity to create
		var	oEntity = entityService.new( prc.mdDictionary.name );
		// Validate entity if possible via ColdBox validation
		if( !coldboxValidation( oEntity, "create", arguments.event, arguments.rc, arguments.prc ) ){ return; }
		// Populate it with RC data
		populateModel( oEntity );
		//Loop Through relations
		for (var i=1; i lte ArrayLen( prc.mdDictionary.singularRelations ); i++ ){
			// check if we have the alias
			if( structKeyExists(rc, "fk_" & prc.mdDictionary.singularRelations[i].alias)
				AND  len( rc["fk_" & prc.mdDictionary.singularRelations[i].alias] ) GT 0 ){
				var tmpEntity = entityService.get(entityName=prc.mdDictionary.singularRelations[i].sourceEntity,
												  id=rc["fk_" & prc.mdDictionary.singularRelations[i].alias]);

				// add the relationship to the entity
				evaluate("oEntity.set#prc.mdDictionary.singularRelations[i].alias#( tmpEntity )");
			}
		}
		// Save entity
		entityService.save( oEntity );
		
		// If the user came from somewhere specific...
		if( len( rc.rURL ) ) {
			// ...go back there.
			setNextEvent( event=rc.rURL );
		} else {
			// Otherwise, relocate back to listing
			setNextEvent(event=prc.xehEntityList, queryString="#rc.entity#");
		}
    }

	/**
    * The entity editor
    */
    function editor(event, rc, prc){
    	    	
		// exit handlers
		prc.xehEntityDelete = "databoss/doDelete";
    	
		//Get Entity's md Dictionary
		prc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );
		// Load the entity requested to edit by ID
		prc.oEntity = entityService.get(rc.entity, rc.id);
		//Get the primary key: TODO, have an invoker method so no evaluate is used
		prc.pkValue = evaluate("prc.oEntity.get#arrayToList(prc.mdDictionary.PK)#()");

		// Check for singular relations
		if ( prc.mdDictionary.hasSingularRelations ){
			//Get Entity Listings
			for (var i=1; i lte ArrayLen( prc.mdDictionary.singularRelations ); i++ ){
				prc["q#prc.mdDictionary.singularRelations[i].alias#"] = entityService.list(entityName=prc.mdDictionary.singularRelations[i].sourceEntity);
			}
		}

		// Check for collection relations
		if( prc.mdDictionary.hasCollections ) {
			//Get Entity Listings, TODO: add sorts
			for (var i=1; i lte ArrayLen( prc.mdDictionary.collections ); i++){
				var tmpAlias = prc.mdDictionary.collections[i].alias;
				// store lookup queries for additions
				prc[ "q#tmpAlias#" ] = entityService.list(entityName=prc.mdDictionary.collections[i].sourceEntity);
				// store the actual array of entities
				prc[ "#tmpAlias#Array" ] = evaluate("prc.oEntity.get#tmpAlias#()");
				// store the collection dictionary
				prc[ "#tmpAlias#Dictionary" ] = metadataService.getDictionaryEntry( prc.mdDictionary.collections[i].sourceEntity );
			}
		}
		
		// Rendering + Exports
		prc.export = false;
		switch( rc.format ){
			case "json" : case "xml" : case "wddx" : case "jsonp" : {
				var eData = queryToStruct( entityToQuery( prc.oEntity ) );
				
				// Do Singular Relations
				if ( prc.mdDictionary.hasSingularRelations ){
					
					// Singular Relations
					for (var i=1; i lte arrayLen( prc.mdDictionary.singularRelations ); i++ ){
						var tmpValue = "";
						if( evaluate("prc.oEntity.has#prc.mdDictionary.singularRelations[i].alias#()") ){
							tmpValue = evaluate("prc.oEntity.get#prc.mdDictionary.singularRelations[i].alias#()");
						}
						if( isObject( tmpValue ) ){
							eData[ prc.mdDictionary.singularRelations[i].alias ] = queryToStruct( entityToQuery( tmpValue ) );
						}
						else{
							eData[ prc.mdDictionary.singularRelations[i].alias ] = tmpValue;
						}
					}
				}
				
				// Check for collection relations
				if( prc.mdDictionary.hasCollections ) {
					for (var i=1; i lte ArrayLen( prc.mdDictionary.collections ); i++){
						var tmpAlias = prc.mdDictionary.collections[i].alias;
						// store the actual array of entities
						eData[ "#tmpAlias#" ] = entityToQuery( evaluate("prc.oEntity.get#tmpAlias#()") );
					}
				}
				
				event.renderData(type=rc.format, 
								 data=eData, 
								 xmlRootName="#rc.entity#", 
								 jsonCallback=getModuleSettings("databoss").settings.jsonpcallback );
				break;
			}
			case "pdf" : {
				prc.export = true;
				event.renderData( type="pdf", data=renderView(view="databoss/editor", module="databoss") );
				break;
			}
			default: {
				//Set view to render
				event.setView("databoss/editor");	
			}
		}
		
    }
    
	/**
    * doUpdate
    */
    function doUpdate(event, rc, prc){
		//Get Entity's md Dictionary
		prc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );
		//Get the entity to update
		var oEntity = entityService.get( prc.mdDictionary.name, rc.id );
		// Validate entity if possible via ColdBox validation
		if( !coldboxValidation( oEntity, "update", arguments.event, arguments.rc, arguments.prc ) ){ return; }
		//Populate it with RC data
		populateModel( oEntity );
		//Loop Through relations
		for ( var i=1;i lte ArrayLen( prc.mdDictionary.singularRelations ); i++ ){

			if( structKeyExists(rc, "fk_" & prc.mdDictionary.singularRelations[i].alias)
				AND  len( rc["fk_" & prc.mdDictionary.singularRelations[i].alias]) GT 0 ){
				var	tmpEntity = entityService.get(entityName=prc.mdDictionary.singularRelations[i].sourceEntity,
												  id=rc["fk_" & prc.mdDictionary.singularRelations[i].alias]);
				//add the tmpTO to oLookup
				evaluate("oEntity.set#prc.mdDictionary.singularRelations[i].alias#( tmpEntity )");
			}

		}
		//Tell service to save object
		entityService.save( oEntity );
		
		// If the user came from somewhere specific...
		if( len( rc.rURL ) ) {
			// ...go back there.
			setNextEvent( event=rc.rURL );
		} else {
			// Otherwise, relocate back to listing
			setNextEvent(event=prc.xehEntityList, queryString="#rc.entity#");
		}
		
    }

	/**
    * doUpdateRelation
    */
    function doUpdateRelation(event, rc, prc){
    	// param values
		var oRelation = oEntity = mdDictionary = relationDictionary = relationSingularName = "";

		// Incoming Args: entity, entityID, addrelation[boolean], relationSource, relationAlias

		//Get Source Entity's md Dictionary
		rc.mdDictionary = metadataService.getDictionaryEntry( rc.entity );
		//Get the entity to update
		oEntity = entityService.get( rc.mdDictionary.name, rc.entityID );
		// Get the relation singular name
		for(var rel in rc.mdDictionary.collections ){
			if( rc.relationSource eq rel.sourceEntity ){
				relationSingularName = rel.singularName;
			}
		}
		
		//Adding or Deleting
		if ( rc.addrelation ){
			//Get the relation object
			oRelation = entityService.get( rc.relationSource, rc[ "collection_#rc.relationAlias#" ] );
			
			//Check if it is already in the collection
			if ( NOT evaluate( "oEntity.has#relationSingularName#(oRelation)" ) ){
				//Add Relation to parent
				evaluate( "oEntity.add#relationSingularName#(oRelation)" );
			}
		}
		else if( structKeyExists( rc, "collection_#rc.relationAlias#_id" ) ){
			var deleteRelationList = rc[ "collection_#rc.relationAlias#_id" ];

			//Remove Relations
			for (var i=1; i lte listlen( deleteRelationList ); i++){
				// Get Relation Object
				oRelation = entityService.get( rc.relationSource, listGetAt( deleteRElationList,i ) );
				// Remove Relation from parent
				evaluate( "oEntity.remove#relationSingularName#( oRelation )" );
			}
		}

		// Persist entity
		entityService.save( oEntity );

		// Relocate back to edit
		setNextEvent(event="databoss.entity.#rc.entity#", queryString="id=#rc.entityID#/collection/true");
    }

    /************************************** PRIVATE *********************************************/

	/**
	* ColdBox Validation
	*/
	private boolean function coldboxValidation(
		oEntity, 
		type, 
		event, 
		rc, 
		prc
	){
		var valid 		= false;
		var constraints = arguments.prc.mdDictionary.insertConstraints;

		// ColdBox direct ORM constraints validation
		if( structKeyExists( arguments.oEntity, "constraints" ) ){
			constraints = arguments.oEntity.constraints;
		}

		// Validate RC
		var validationResults = validateModel( target=arguments.rc, constraints=constraints );
		if( validationResults.hasErrors() ){
			// MB for error
			getPlugin( "MessageBox" ).error( messageArray=validationResults.getAllErrors() );
		} else {
			valid = true;
		}

		// execute method to show back
		if( !valid ){
			if( arguments.type eq "create" ){
				create( arguments.event, arguments.rc, arguments.prc );
			} else {
				editor( arguments.event, arguments.rc, arguments.prc );
			}
		}

		return valid;
	}


	/**
	* Get the sorted entities
	*/
	private Array function getSortedEntities(entities){
		var sortedEntities = '';

		sortedEntities = structKeyArray(arguments.entities);
		ArraySort(sortedEntities, "text");

		return sortedEntities;
	}
	
	 // Utility function
    private function queryToStruct(target){
    	var columns = listToArray( arguments.target.columnlist );
    	var result = {};
    	for( var c in columns ){
    		result[ c ] = arguments.target[ c ][ 1 ];
    	}
    	return result;
    }
}