component singleton {
	
	function buildDisplayColumns( required entity, required relatedPropertyDictionary ) {
		var relAlias = relatedPropertyDictionary.alias;
		var relDisplayColumns = relatedPropertyDictionary.displayColumns;
		var relPK = relatedPropertyDictionary.pk; 
		
		var thisValue = '<null>';
		
		// If relationship is empty, just return null
		if( !evaluate( "entity.has#relAlias#()" ) ) {
			return thisValue;
		}
			
		var relatedEntity = evaluate( "entity.get#relAlias#()" );
		
		// Check if no DisplayColumns, else use PK
		if( !len( relDisplayColumns ) ) {
			return evaluate( "relatedEntity.get#arrayToList( relPK )#()" );		
		}
		
		// We have displayColumns, so let's build them up in this array!
		var aThisValue = [];
		
		// For each column...
		for( var col in listToArray( relDisplayColumns ) ) {

			// This will be our moving target.  Once the while loop below is done, it should hold the final value 
			thisColumnData = relatedEntity;
			
			// Each column *may* be a dot-delimited list to allow reference to "deep" properties
			// i.e. db_displayColumns="supervisor.company.name";
			for( var colPart in listToArray( col, '.' ) ) {
				
				thisColumnData = evaluate( "thisColumnData.get#colPart#()" );
					
				// If we've reached a dead end, stop 
				if( isNull( thisColumnData ) ) {
					thisColumnData = '<null>';
					// Break out of this column, but the outer loop will keep going on the next column
					break;
				}
				
			}
			
			// Add this column to the array
			arrayAppend( aThisValue, thisColumnData );
						
		}
		
		// Turn the array of values back into a list
		thisValue = arrayToList( aThisValue, ", " );
				
		// Nullable check
		if( isNull( thisValue ) ) {
			thisValue = '<null>'; 
		}

		return thisValue;

	}
	
}